/*
 * Author und Copyright: Ruediger Berlich
 *
 * Distributed under the Boost Software License, Version 1.0. 
 * See http://www.boost.org/LICENSE_1_0.txt
 *
 * Run with the command "root -l dataInput.cpp" .
 * Requires a ROOT-version >= 6
 * Tested with ROOT 6.02.02 on MacOS X Yosemite
 */

#include <memory>
#include <functional>
#include <random>
#include <tuple>
#include <cmath>
#include <iostream>

typedef std::mt19937 GENERATOR;

/**
 * Retrieve a random number that is distributed according to a given function.
 * No error checks are done, for reasons of simplicity.
 */
double getRandomNumberWithDistribution (
  const std::function<double(double)>& f
  , GENERATOR& generator
  , double x_min, double x_max
  , double y_min, double y_max
) {
  double x=0., y=0.;
  
  // The random distributions
  std::uniform_real_distribution<double> distribution_x(x_min, x_max);
  std::uniform_real_distribution<double> distribution_y(y_min, y_max);
  
  // Retrieve random variables in the allowed x- and y-range until y is < f(x)
  do {
    x = distribution_x(generator);
    y = distribution_y(generator);
  } while(f(x) < y);
  
  return x;
}

const double MINAGE=18, MAXAGE=88;

/**
 * The "main()"-function (for ROOT: A function with the same name as the file)
 */
void dataInput(void) {
  // This will hold the generated data
  TNtuple *moneySpent = new TNtuple(
    "moneySpent"
    ,"Money spent"
    ,"moneySpentLastDay:age:season"
  );

  // Create a C++11-style random number generator.
  std::random_device rand_dev;          // Needed to get a good random seed
  GENERATOR generator(rand_dev());

  // Set the number of data sets for each season in the order 
  // spring, summer, fall and winter
  std::vector<std::size_t> nDataSeason = {12000, 9000, 11000, 8000};

  // Older people are more likely than younger people (up to a certain age ...)
  std::function<double(double)> ageDist_f = [=](double x) -> double { 
    return (x<=70?1.+0.01*x:(1+0.01*70)-0.06*(x-70));
  };

  // Create the datasets
  std::size_t season, season_pos= 0;
  for(std::size_t nData: nDataSeason) { // Loop over all seasons
  	season = season_pos++; // Determine the season
		  	for(std::size_t interview=0; interview < nData; interview++) {
				// Determine the age of the interviewees
		  	   double age 	
		  	   = getRandomNumberWithDistribution(
  	    		  ageDist_f
			      , generator
		    	  , MINAGE, MAXAGE
			      , 0., 2
		  	   );  
		  	   
		  	   // Find out about the money spent by the interviewee. We assume
		  	   // that people at the age of 70 are in average able to spent
		  	   // twice as much as at the age of 18, and that thereafter the 
		  	   // amount decreases again. We model this through the mean of a gaussian.
			   std::normal_distribution<double> gauss(
			     age<=70?(10.*(1.+(age-18.)/(70.-18.))):(20.*(1.-0.5*(age-70.)/(88.-70.))) 
			     ,5.
			    );
			   // Make sure we do not get negative values
		  	   double moneySpentLastDay = fabs(gauss(generator));
		  	   
		  	   // Fill the NTuple
		  	   moneySpent->Fill(moneySpentLastDay, age, double(season));
		  	}
  }
  
  // Save the NTuple to a file
  TFile *f = new TFile("ageDistribution.root", "RECREATE");
  moneySpent->Write();
  f->Close();
}
